package de.chaubold.gleshelper;

import de.chaubold.math.Vec3;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.util.Vector;
import javax.microedition.khronos.opengles.GL10;


/**
 * A VertexBufferObject which sends the vertices and normals to the RAM of the graphics card if VBOs are supported, otherwise falls back to VertecArrays
 * @author Carsten Haubold, 17 Jan 2008
 */
public class VBO
{
	private FloatBuffer vertices = null;
	private FloatBuffer normals = null;
	private int numFaces=0;
	private Vector<Vec3> vertexData = new Vector<Vec3>();
	private Vector<Vec3> normalData = new Vector<Vec3>();
	private boolean finalized=false;
	
	public VBO()
	{
		
	}
	
	private void CheckError(GL10 gl, String debug) {
        if (gl.glGetError() != gl.GL_NO_ERROR) {
            System.out.println("Error at " + debug);
        }
    }
	
	public void Draw(GL10 gl, int start, int end)
	{
		if(!finalized)
			createVertexBuffer();
		// invoke renderer to draw the needed vertices
		gl.glEnableClientState(gl.GL_VERTEX_ARRAY);
		gl.glVertexPointer(3,GL10.GL_FLOAT, 0, vertices);
		CheckError(gl,"VertexPointer");
		gl.glEnableClientState(gl.GL_NORMAL_ARRAY);
		gl.glNormalPointer(0,GL10.GL_FLOAT, normals);
		CheckError(gl,"NormalPointer");
		gl.glDrawArrays(gl.GL_TRIANGLES,start,end-start);
		CheckError(gl,"DrawArrays");
		gl.glDisableClientState(gl.GL_NORMAL_ARRAY);
		gl.glDisableClientState(gl.GL_VERTEX_ARRAY);
	}
	
	private void createVertexBuffer()
	{
        ByteBuffer vbb = ByteBuffer.allocateDirect(vertexData.size()*3);
        vbb.order(ByteOrder.nativeOrder());
		this.vertices=vbb.asFloatBuffer();
		for(int x=0;x<vertexData.size();x++)
		{
			vertices.put(x*3+0,vertexData.get(x).x);
			vertices.put(x*3+1,vertexData.get(x).y);
			vertices.put(x*3+2,vertexData.get(x).z);
		}
		vertexData.clear();
        vertices.position(0);
		
		ByteBuffer nbb = ByteBuffer.allocateDirect(normalData.size()*3);
        nbb.order(ByteOrder.nativeOrder());
		this.normals=nbb.asFloatBuffer();
		for(int x=0;x<normalData.size();x++)
		{
			normals.put(x*3+0,normalData.get(x).x);
			normals.put(x*3+1,normalData.get(x).y);
			normals.put(x*3+2,normalData.get(x).z);
		}
		normalData.clear();
        normals.position(0);
				
		finalized=true;
	}
	
	public int getCurrIndex()
	{
		return vertexData.size();
	}
	
	public void addTriangle(Vec3 a,Vec3 na,Vec3 b,Vec3 nb,Vec3 c,Vec3 nc)
	{
		vertexData.add(a);
		normalData.add(na);
		vertexData.add(b);
		normalData.add(nb);
		vertexData.add(c);
		normalData.add(nc);
		numFaces++;
	}
	
	public void addTriangle(Vec3 a,Vec3 b,Vec3 c)
	{
		Vec3 normal=Vec3.normal(a,b,c);
		
		vertexData.add(a);
		normalData.add(normal);
		vertexData.add(b);
		normalData.add(normal);
		vertexData.add(c);
		normalData.add(normal);
		numFaces++;
	}
}
