package de.chaubold.gleshelper;

import de.chaubold.math.Vec2;
import de.chaubold.math.Vec3;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.util.Vector;
import javax.microedition.khronos.opengles.GL10;


/**
 * A VertexBufferObject which sends the vertices, textureCoords and normals to the RAM of the graphics card if VBOs are supported, otherwise falls back to VertecArrays
 * @author Carsten Haubold, 17 Jan 2008
 */
class VBOwithTex
{
	private FloatBuffer vertices = null;
	private FloatBuffer normals = null;
    private FloatBuffer texCoords = null;
	private int numFaces=0;
	private Vector<Vec3> vertexData = new Vector<Vec3>();
	private Vector<Vec3> normalData = new Vector<Vec3>();
    private Vector<Vec2> texCoordData = new Vector<Vec2>();
	private boolean finalized=false;

	public VBOwithTex()
	{

	}

	private void CheckError(GL10 gl, String debug) {
        if (gl.glGetError() != GL10.GL_NO_ERROR) {
            System.out.println("Error at " + debug);
        }
    }

	public void Draw(GL10 gl, int start, int end)
	{
		if(!finalized)
			createVertexBuffer();
		// invoke renderer to draw the needed vertices
		gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
		gl.glVertexPointer(3,GL10.GL_FLOAT, 0, vertices);
		CheckError(gl,"VertexPointer");
		gl.glEnableClientState(GL10.GL_NORMAL_ARRAY);
		gl.glNormalPointer(0,GL10.GL_FLOAT, normals);
		CheckError(gl,"NormalPointer");

        //ARBMultitexture.glClientActiveTextureARB(ARBMultitexture.GL_TEXTURE0_ARB);
		gl.glEnableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
		gl.glTexCoordPointer(2,GL10.GL_FLOAT, 0, texCoords);
		CheckError(gl,"TexCoordPointer");

		gl.glDrawArrays(gl.GL_TRIANGLES,start,end-start);
		CheckError(gl,"DrawArrays");
		gl.glDisableClientState(GL10.GL_NORMAL_ARRAY);
		gl.glDisableClientState(GL10.GL_VERTEX_ARRAY);
	}

	private void createVertexBuffer()
	{
        ByteBuffer vbb = ByteBuffer.allocateDirect(vertexData.size()*3);
        vbb.order(ByteOrder.nativeOrder());
		this.vertices=vbb.asFloatBuffer();
		for(int x=0;x<vertexData.size();x++)
		{
			vertices.put(x*3+0,vertexData.get(x).x);
			vertices.put(x*3+1,vertexData.get(x).y);
			vertices.put(x*3+2,vertexData.get(x).z);
		}
		vertexData.clear();
        vertices.position(0);

		ByteBuffer nbb = ByteBuffer.allocateDirect(normalData.size()*3);
        nbb.order(ByteOrder.nativeOrder());
		this.normals=nbb.asFloatBuffer();
		for(int x=0;x<normalData.size();x++)
		{
			normals.put(x*3+0,normalData.get(x).x);
			normals.put(x*3+1,normalData.get(x).y);
			normals.put(x*3+2,normalData.get(x).z);
		}
		normalData.clear();
        normals.position(0);

        ByteBuffer tbb = ByteBuffer.allocateDirect(texCoordData.size()*2);
        tbb.order(ByteOrder.nativeOrder());
		this.texCoords = tbb.asFloatBuffer();
		for(int x=0;x<texCoordData.size();x++)
		{
			texCoords.put(x*2+0,texCoordData.get(x).x);
			texCoords.put(x*2+1,texCoordData.get(x).y);
		}
		texCoordData.clear();
        texCoords.position(0);

		finalized=true;
	}

	public int getCurrIndex()
	{
		return vertexData.size();
	}

	public void addTriangle(Vec3 a,Vec2 ta,Vec3 na,Vec3 b,Vec2 tb,Vec3 nb,Vec3 c,Vec2 tc,Vec3 nc)
	{
		vertexData.add(a);
        texCoordData.add(ta);
		normalData.add(na);
		vertexData.add(b);
		texCoordData.add(tb);
        normalData.add(nb);
		vertexData.add(c);
        texCoordData.add(tc);
		normalData.add(nc);
		numFaces++;
	}

	public void addTriangle(Vec3 a,Vec2 ta,Vec3 b,Vec2 tb,Vec3 c, Vec2 tc)
	{
		Vec3 normal=Vec3.normal(a,b,c);

		vertexData.add(a);
        texCoordData.add(ta);
		normalData.add(normal);
		vertexData.add(b);
        texCoordData.add(tb);
		normalData.add(normal);
		vertexData.add(c);
        texCoordData.add(tc);
		normalData.add(normal);
		numFaces++;
	}
}

