/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package de.chaubold.math;

/**
 *
 * @author caste
 */
public class Vec3 {

    public float x = 0.0f,  y = 0.0f,  z = 0.0f;

    /** Simply creates instance, components initialised to
    <tt>(float)0.0</tt> */
    public Vec3() {
        //vector already set to (0.0,0.0,0.0)
    }

    public Vec3(Vec3 a) {
        x = a.x;
        y = a.y;
        z = a.z;
    }

    public Vec3(float xx1, float xx2, float xx3) {
        x = xx1;
        y = xx2;
        z = xx3;
    }

    public void setValue(float xx1, float xx2, float xx3) {
        x = xx1;
        y = xx2;
        z = xx3;
    }

    public void setValue(Vec3 a) {
        x = a.x;
        y = a.y;
        z = a.z;
    }

    /** Sets <tt>Vec3 this</tt> to zero */
    public void setZero() {
        x = (float) 0.0;
        y = (float) 0.0;
        z = (float) 0.0;
    }

    public Vec3 add(Vec3 a) {
        return new Vec3(x + a.x, y + a.y, z + a.z);
    }

    public Vec3 sub(Vec3 a) {
        return new Vec3(x - a.x, y - a.y, z - a.z);
    }

    public static float dot(Vec3 a, Vec3 b) {
        return (a.x * b.x + a.y * b.y + a.z * b.z);
    }

    public static Vec3 cross(Vec3 a, Vec3 b) {
        Vec3 out = new Vec3();
        out.x = a.y * b.z - a.z * b.y;
        out.y = a.z * b.x - a.x * b.z;
        out.z = a.x * b.y - a.y * b.x;
        return out;
    }

    /** Calculate unit normal to triangle given by position vectors a, b
    and c in the sense that a->b->c goes right-handed around n. For
    example, if a, b and c are clockwise and flat in the plane
    of view, n is away from the viewer.<p>
    Alternatively, this is (b-a) cross (c-a) normalised.
     */
    public static Vec3 normal(Vec3 a, Vec3 b, Vec3 c) {
        Vec3 out = new Vec3();
        out.x = (b.y - a.y) * (c.z - a.z) - (b.z - a.z) * (c.y - a.y);
        out.y = (b.z - a.z) * (c.x - a.x) - (b.x - a.x) * (c.z - a.z);
        out.z = (b.x - a.x) * (c.y - a.y) - (b.y - a.y) * (c.x - a.x);
        return out.normalize();
    }

    public Vec3 normalize() {
        Vec3 out = new Vec3();
        float length;
        length = length();
        out.x = x / length;
        out.y = y / length;
        out.z = z / length;
        return out;
    }

    /** Returns length of vector */
    public float length() {
        return ((float) Math.sqrt(x * x + y * y + z * z));
    }

    /** Returns length of vector squared */
    public float lengthSqr() {
        return (x * x + y * y + z * z);
    }

    /** Returns length of vector (b-a) squared */
    public static float distSqr(Vec3 a, Vec3 b) {
        return ((float) (a.x - b.x) * (a.x - b.x) +
                (a.y - b.y) * (a.y - b.y) +
                (a.z - b.z) * (a.z - b.z));
    }

    public Vec3 mult(float a)
    {
        return new Vec3(a*x, a*y, a*z);
    }
}
