/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package de.chaubold.physicstest.physic;

import de.chaubold.math.Vec2;
import java.util.Vector;

/**
 *
 * @author caste
 */
public class Polygon {
    int _type;
    Vector<Particle> _vertices = new Vector<Particle>();
    Vector<Constraint> _edges = new Vector<Constraint>();
    Vec2 _center = new Vec2();

    public Polygon(Vector<Constraint> edges, Vector<Particle> vertices, int type) {
        _edges = edges;
        _vertices = vertices;
        _type = type;
        update();
    }

    public boolean checkCollisionAgainst(Polygon other, boolean performCollisionResponse) {
        //perform "Separating axis theorem"
        Constraint currEdge;
        Vec2 axis = new Vec2();
        Vec2 proj1, proj2;
        float minDistance = Float.MAX_VALUE;
        Vec2 minAxis = new Vec2();
        Constraint minEdge = null;//store the edge which gets penetrated by the other object

        for (int i = 0; i < _edges.size() + other._edges.size(); i++) {
            //Select current Edge
            if (i < _edges.size()) {
                currEdge = _edges.get(i);
            } else {
                currEdge = other._edges.get(i - _edges.size());
            }

            axis.x = -(currEdge._p2._position.y - currEdge._p1._position.y);
            axis.y = (currEdge._p2._position.x - currEdge._p1._position.x);
            axis = axis.normalize();

            proj1 = projectToAxis(axis);
            proj2 = other.projectToAxis(axis);

            float dist = interpenetrationDepth(proj1, proj2);
            if (dist > 0) {
                return false;
            } else if (Math.abs(dist) < minDistance) {
                minDistance = Math.abs(dist);
                minAxis.setValue(axis);
                minEdge = currEdge;
            }
        }

        if (performCollisionResponse) {
            //axis is the vector specifying the direction and amount of interpenetration
            axis.setValue(minAxis.mult(minDistance));

            //look for the interpenetrating point in Polygon p
            Polygon p = this;
            if (_edges.contains(minEdge)) {
                p = other;
            }
            //use the hesseian normal form
            float c = -p._center.x * minAxis.x - p._center.y * minAxis.y;
            minDistance = Float.MAX_VALUE;
            Particle penetratingPoint = null;
            float d;
            
            if(Vec2.dot(p._center, minAxis) + c <= 0)
            {
                minAxis.setValue(minAxis.mult(-1.0f));
                axis.setValue(axis.mult(-1.0f));
            }

            for (Particle v : p._vertices) {
                d = Vec2.dot(v._position, minAxis) + c;
                if (d < minDistance) {
                    minDistance = d;
                    penetratingPoint = v;
                }
            }

            penetratingPoint._position.setValue(penetratingPoint._position.add(axis.mult(0.5f)));
            float ratio = 0.0f;

            float d1 = minEdge._p1._position.x - minEdge._p2._position.x;
            float d2 = minEdge._p1._position.y - minEdge._p2._position.y;

            if(Math.abs(d1) > Math.abs(d2))
            {
                ratio = (penetratingPoint._position.x - axis.x - minEdge._p1._position.x) / (-d1);
            }
            else
            {
                if(d2 != 0.0)
                {
                    ratio = (penetratingPoint._position.y - axis.y - minEdge._p1._position.y) / (-d2);
                }
            }

            minEdge._p1._position.setValue(minEdge._p1._position.add(axis.mult((1.0f-ratio)*-0.5f)));
            minEdge._p2._position.setValue(minEdge._p2._position.add(axis.mult(-ratio*0.5f)));
        }

        return true;
    }

    //The returned vector contains the minimal value
    private Vec2 projectToAxis(Vec2 axis) {
        float min, max;
        min = max = Vec2.dot(axis, _vertices.get(0)._position);

        for (int i = 1; i < _vertices.size(); i++) {
            float dot = Vec2.dot(axis, _vertices.get(i)._position);

            if (dot > max) {
                max = dot;
            } else if (dot < min) {
                min = dot;
            }
        }

        return new Vec2(min, max);
    }

    private float interpenetrationDepth(Vec2 proj1, Vec2 proj2) {
        if (proj1.x < proj2.x) {
            return proj2.x - proj1.y;
        } else {
            return proj1.x - proj2.y;
        }
    }

    public static Polygon createBox(System system, Vec2 position, Vec2 size, float stiffness) {
        Vector<Constraint> edges = new Vector<Constraint>();
        Vector<Particle> vertices = new Vector<Particle>();

        Particle p1 = new Particle(position.add(new Vec2(size.x * 0.5f, size.y * 0.5f)), system);
        Particle p2 = new Particle(position.add(new Vec2(-size.x * 0.5f, size.y * 0.5f)), system);
        Particle p3 = new Particle(position.add(new Vec2(-size.x * 0.5f, -size.y * 0.5f)), system);
        Particle p4 = new Particle(position.add(new Vec2(size.x * 0.5f, -size.y * 0.5f)), system);

        vertices.add(p1);
        vertices.add(p2);
        vertices.add(p3);
        vertices.add(p4);

        system._particles.addAll(vertices);

        edges.add(new Constraint(p1, p2, stiffness));
        edges.add(new Constraint(p2, p3, stiffness));
        edges.add(new Constraint(p3, p4, stiffness));
        edges.add(new Constraint(p4, p1, stiffness));
        edges.add(new Constraint(p1, p3, stiffness));
        edges.add(new Constraint(p4, p2, stiffness));

        system._constraints.addAll(edges);

        return new Polygon(edges, vertices,1);
    }

    public static Polygon createBox(System system, Vec2 position, Vec2 size) {
        return createBox(system, position, size, 1.0f);
    }

    public static Polygon createTriangle(System system, Vec2 position, Vec2 size, float stiffness) {
        Vector<Constraint> edges = new Vector<Constraint>();
        Vector<Particle> vertices = new Vector<Particle>();

        Particle p1 = new Particle(position.add(new Vec2(size.x * 0.5f, size.y * 0.5f)), system);
        Particle p2 = new Particle(position.add(new Vec2(-size.x * 0.5f, size.y * 0.5f)), system);
        Particle p3 = new Particle(position.add(new Vec2(0.0f, -size.y * 0.5f)), system);

        vertices.add(p1);
        vertices.add(p2);
        vertices.add(p3);

        system._particles.addAll(vertices);

        edges.add(new Constraint(p1, p2, stiffness));
        edges.add(new Constraint(p2, p3, stiffness));
        edges.add(new Constraint(p1, p3, stiffness));

        system._constraints.addAll(edges);

        return new Polygon(edges, vertices,2);
    }

    public static Polygon createTriangle(System system, Vec2 position, Vec2 size) {
        return createTriangle(system, position, size, 1.0f);
    }

    public void update() {
        _center.setValue(0.0f, 0.0f);
        for (Particle p : _vertices) {
            _center.setValue(_center.add(p._position));
        }
        _center = _center.mult(1.0f / _vertices.size());
    }
}
