/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package de.chaubold.physicstest.physic;

import de.chaubold.math.Vec2;
import de.chaubold.math.Vec3;
import java.util.Vector;

/**
 *
 * @author caste
 */
public class System {
    Vec2 _gravity;
    Vector<Particle> _particles = new Vector<Particle>();
    Vector<Constraint> _constraints = new Vector<Constraint>();
    Vector<Polygon> _polygons = new Vector<Polygon>();
    long _lastTime;

    final int _iterations = 1;

    Vec2 _worldDimension;

    public System()
    {
        _gravity = new Vec2(0.0f, -1.0f);
        _worldDimension = new Vec2(9.0f, 10.0f);
        initializeSystem();
    }

    public void setGravityVector(Vec3 grav) {
        _gravity.x = 9.0f*grav.x;
        _gravity.y = 9.0f*grav.y;
    }

    public void update()
    {
        long currTime = android.os.SystemClock.elapsedRealtime();
        float dT = 0.001f * (float)(currTime - _lastTime);
        if(dT > 0.01f)
        {
            dT = 0.01f;
        }
        _lastTime = currTime;

        //Update particles
        for(Particle p : _particles)
        {
            p.update(dT);
        }

        //Update Constraints
        for(int i = 0; i < _iterations; i++)
        {
            for(Constraint c : _constraints)
            {
                c.update();
            }
        }

        for(Polygon p : _polygons)
        {
            p.update();
        }

        //Check for collisions
        for(int i = 0; i < _polygons.size(); i++)
        {
            for(int j = i+1; j< _polygons.size(); j++)
            {
                _polygons.get(i).checkCollisionAgainst(_polygons.get(j), true);
            }
        }

    }

    public void resetTime()
    {
        _lastTime = android.os.SystemClock.elapsedRealtime();
    }

    private void initializeSystem() {
        _polygons.add(Polygon.createBox(this, new Vec2(0.0f,2.0f), new Vec2(3.0f,4.0f)));
        _polygons.add(Polygon.createBox(this, new Vec2(2.0f,5.0f), new Vec2(2.0f,2.0f)));
        _polygons.add(Polygon.createBox(this, new Vec2(-2.0f,5.5f), new Vec2(2.0f,2.0f)));
        _polygons.add(Polygon.createBox(this, new Vec2(0.0f,6.0f), new Vec2(1.0f,1.0f)));
        _lastTime = android.os.SystemClock.elapsedRealtime();
    }
}
